@@ -13,7 +13,7 @@ def rook_test():
         LoggingConfiguration.LOG_TO_STDERR = True
 
     from rook.config import AgentComConfiguration
-    AgentComConfiguration.TIMEOUT = 10
+    AgentComConfiguration.TIMEOUT = AgentComConfiguration.PING_TIMEOUT + 1
 
     try:
         import rook
