@@ -75,7 +75,9 @@ class AugFactory(Factory):
             limits_spec = configuration.get('rateLimit', "")
             quota = utils.milliseconds_to_int_nano_seconds(200)
             window_size = utils.milliseconds_to_int_nano_seconds(5000)
-            limiters.append(self.create_rate_limiter(limits_spec, quota, window_size))
+            limiter = self.create_rate_limiter(limits_spec, quota, window_size)
+            if limiter:
+                limiters.append(limiter)
 
         max_aug_execution_time = AugFactory.get_dict_value(configuration, 'maxAugTime', config.InstrumentationConfig.MAX_AUG_TIME_MS)
         max_aug_execution_time = utils.milliseconds_to_int_nano_seconds(max_aug_execution_time)
