@@ -74,27 +74,26 @@ class Output(object):
         if not self._agent_com:
             return
 
-        def send_msg():
-            if arguments and arguments.size(''):
-                try:
-                    if config.DumpConfig.PROTOBUF_VERSION_2:
-                        self._agent_com.send_user_message(aug_id, message_id, arguments)
-                    else:
-                        msg = AugReportMessage()
-                        msg.agent_id = self._id
-                        msg.aug_id = aug_id
-                        msg.report_id = str(message_id)
+        def send_msg():        
+            try:
+                if config.DumpConfig.PROTOBUF_VERSION_2:
+                    self._agent_com.send_user_message(aug_id, message_id, arguments)
+                else:
+                    msg = AugReportMessage()
+                    msg.agent_id = self._id
+                    msg.aug_id = aug_id
+                    msg.report_id = str(message_id)
 
-                        serializer = NamespaceSerializer(config.DumpConfig.STRING_CACHE_USERMESSAGE)
-                        serializer.dump(arguments, msg.arguments)
+                    serializer = NamespaceSerializer(config.DumpConfig.STRING_CACHE_USERMESSAGE)
+                    serializer.dump(arguments, msg.arguments)
 
-                        for k, v in six.iteritems(serializer.get_string_cache()):
-                            msg.strings_cache[k] = v
+                    for k, v in six.iteritems(serializer.get_string_cache()):
+                        msg.strings_cache[k] = v
 
-                        self._agent_com.add(msg)
-                        self.skipped_aug_ids.remove(aug_id)
-                except RookOutputQueueFull:
-                    self.send_output_queue_full_warning(aug_id)
+                    self._agent_com.add(msg)
+                    self.skipped_aug_ids.remove(aug_id)
+            except RookOutputQueueFull:
+                self.send_output_queue_full_warning(aug_id)
 
         self._user_message_bucket.do_if_available(send_msg)
 
