@@ -1,3 +1,6 @@
+from rook import config
+
+
 class ToolException(Exception):
 
     def __init__(self, *args):
@@ -201,22 +204,29 @@ class RookOperationReadOnly(RookMonitorException):
                                                     {'operation': operation_type.__name__})
 
 
-class RookRuleRateLimited(RookException):
+class RookRuleAugRateLimited(RookException):
     def __init__(self):
-        super(RookRuleRateLimited, self).__init__("Breakpoint was disabled due to rate-limiting. "
-                                                  "For more information: https://docs.rookout.com/docs/breakpoints-tasks.html#rate-limiting")
+        super(RookRuleAugRateLimited, self).__init__("Breakpoint was disabled due to rate-limiting. "
+                                                     "For more information: https://docs.rookout.com/docs/breakpoint-limits#rate-limiting")
 
 
-class RookGlobalRuleRateLimited(RookException):
+class RookRuleGlobalRateLimited(RookException):
     def __init__(self):
-        super(RookGlobalRuleRateLimited, self).__init__("Breakpoint was disabled due to global-rate-limiting. "
-                                                        "For more information: https://docs.rookout.com/docs/breakpoints-tasks.html#rate-limiting")
+        super(RookRuleGlobalRateLimited, self).__init__("Breakpoint was disabled due to global-rate-limiting. "
+                                                        "For more information: https://docs.rookout.com/docs/breakpoint-limits#rate-limiting")
+
+
+def rook_rule_rate_limited():
+    if config.RateLimiter.USING_GLOBAL_RATE_LIMITER:
+        return RookRuleGlobalRateLimited()
+    return RookRuleAugRateLimited()
 
 
 class RookInvalidRateLimitConfiguration(RookException):
     def __init__(self, config):
         super(RookInvalidRateLimitConfiguration, self).__init__("Got an invalid value for the rate limit. (got " +
-                                                                config + ") expected XX/YY or XX\\YY, where XX < YY")
+                                                                config + ") expected XX/YY or XX\\YY, where XX < YY",
+                                                                {'config': config})
 
 
 class RookRuleMaxExecutionTimeReached(RookException):
@@ -344,11 +354,11 @@ class RookMessageSizeExceeded(RookException):
 class RookQueueSizeExceeded(RookException):
     def __init__(self, message_size, queue_size, max_queue_size):
         super(RookQueueSizeExceeded, self).__init__("Message size of %d would exceed max queue size of %d. "
-                                                      "Consider changing the depth of collection and trying again." %
-                                                      (message_size, max_queue_size),
-                                                      {'message_size': message_size,
-                                                       'queue_messages_length': queue_size,
-                                                       'max_queue_messages_length': max_queue_size})
+                                                    "Consider changing the depth of collection and trying again." %
+                                                    (message_size, max_queue_size),
+                                                    {'message_size': message_size,
+                                                     'queue_messages_length': queue_size,
+                                                     'max_queue_messages_length': max_queue_size})
 
 
 class RookOutputQueueFull(RookException):
