@@ -8,7 +8,7 @@ import rook.protobuf.variant_pb2
 
 
 class Error(object):
-    def __init__(self, exc=None, type=None, message=None, parameters=None, traceback=None):
+    def __init__(self, exc=None, message=None, parameters=None, traceback=None):
         self.exc = exc
 
         if isinstance(exc, ToolException):
@@ -16,7 +16,7 @@ class Error(object):
             self.message = exc.get_message()
             self.parameters = exc.get_parameters()
         else:
-            self.type = type or "Unknown"
+            self.type = "Unknown"
             self.message = message
             self.parameters = parameters
 
@@ -34,31 +34,32 @@ class Error(object):
                 error.message = NamespaceSerializer.normalize_string(self.message)
             else:
                 error.message = ''
-        except Exception: # We are already deep in error handling code - just ignore
+        except Exception:  # We are already deep in error handling code - just ignore
             pass
 
         try:
             error.type = NamespaceSerializer.normalize_string(self.type)
-        except Exception: # We are already deep in error handling code - just ignore
+        except Exception:  # We are already deep in error handling code - just ignore
             pass
 
         try:
             serializer.dump(PythonObjectNamespace(self.parameters), error.parameters, False)
-        except Exception: # We are already deep in error handling code - just ignore
+        except Exception:  # We are already deep in error handling code - just ignore
             pass
 
         try:
             serializer.dump(PythonObjectNamespace(self.exc), error.exc, False)
-        except Exception: # We are already deep in error handling code - just ignore
+        except Exception:  # We are already deep in error handling code - just ignore
             pass
 
         try:
             if self.traceback:
                 traceback_string = '\n'.join(traceback.format_tb(self.traceback))
                 serializer.dump(PythonObjectNamespace(traceback_string,
-                                                      PythonObjectNamespace.ObjectDumpConfig.tailor_limits(traceback_string)),
+                                                      PythonObjectNamespace.ObjectDumpConfig.tailor_limits(
+                                                          traceback_string)),
                                 error.traceback, False)
-        except Exception: # We are already deep in error handling code - just ignore
+        except Exception:  # We are already deep in error handling code - just ignore
             pass
 
     def dumps(self):
