@@ -11,7 +11,8 @@ from rook.processor.namespace_serializer_base import NamespaceSerializerBase
 
 from .namespaces.container_namespace import ContainerNamespace
 from .namespaces.python_object_namespace import PythonObjectNamespace
-from .namespaces.collection_namespace import DictNamespace, ListNamespace, LIST_TYPE
+from .namespaces.collection_namespace import DictNamespace, ListNamespace
+from .namespaces.variable_types import LIST_TYPE
 from .namespaces.traceback_namespace import TracebackNamespace
 from .namespaces.user_object_namespace import UserObjectNamespace
 from .namespaces.error_namespace import ErrorNamespace
@@ -147,13 +148,13 @@ class NamespaceSerializer(NamespaceSerializerBase):
             self._dump_exception(obj, variant, current_depth, config, log_errors)
         elif isinstance(obj, TracebackType):
             self._dump_traceback(obj, variant, current_depth, config, log_errors)
-        elif NamespaceSerializer.is_numpy_obj(obj):
+        elif NamespaceSerializer.is_numpy_obj(type(obj).mro()):
             self._dump_primitive(obj.item(), str(type(obj)), variant, config.max_string)
-        elif NamespaceSerializer.is_torch_obj(obj):
+        elif NamespaceSerializer.is_torch_obj(type(obj)):
             self._dump_primitive(str(obj), str(type(obj)), variant, config.max_string)
-        elif NamespaceSerializer.is_multidict_obj(obj):
+        elif NamespaceSerializer.is_multidict_obj(type(obj)):
             self._dump_primitive(str(obj), str(type(obj)), variant, config.max_string)
-        elif NamespaceSerializer.is_protobuf_obj(obj):
+        elif NamespaceSerializer.is_protobuf_obj(type(obj)):
             self._dump_protobuf(obj, variant, current_depth, config, log_errors)
         elif hasattr(obj, '__dict__'):
             self._dump_user_class(obj, variant, current_depth, config)
@@ -302,7 +303,7 @@ class NamespaceSerializer(NamespaceSerializerBase):
     def _dump_list(self, collection, variant, current_depth, config, log_errors):
         variant.variant_type = variant.VARIANT_LIST
         variant.list_value.type = ListNamespace.get_common_type(collection)
-        if ListNamespace.is_numpy_obj(collection):
+        if self.is_numpy_array(type(collection)):
             collection = collection.tolist()
             if not collection:
                 collection = []
