@@ -12,7 +12,7 @@ from rook.processor.namespace_serializer_base import NamespaceSerializerBase
 
 from .namespaces.container_namespace import ContainerNamespace
 from .namespaces.python_object_namespace import PythonObjectNamespace
-from .namespaces.collection_namespace import ListNamespace, LIST_TYPE
+from .namespaces.variable_types import LIST_TYPE, DICT_TYPE
 from .namespaces.traceback_namespace import TracebackNamespace
 from .namespaces.error_namespace import ErrorNamespace
 from .namespaces.formatted_namespace import FormattedNamespace
@@ -98,34 +98,34 @@ class NamespaceSerializer2(NamespaceSerializerBase):
                 UserWarnings.send_warning(Error(exc=e, message=message))
 
     def _dump_python_object_unsafe(self, obj, variant, current_depth, config, log_errors):
-        original_type = type(obj).__name__
-        variant.original_type_index_in_cache = self._get_string_index_in_cache(original_type)
+        obj_class = type(obj)
+        obj_mro = object.__getattribute__(obj_class, '__mro__')
+        variant.original_type_index_in_cache = self._get_string_index_in_cache(obj_class.__name__)
         self.estimated_pending_bytes += 3  # Header + number
 
-        if isinstance(obj, NamespaceSerializerBase.PRIMITIVE_TYPES):
-            self._dump_primitive(obj, original_type, variant, config.max_string)
-        elif isinstance(obj, LIST_TYPE):
+        if obj_class in NamespaceSerializerBase.PRIMITIVE_TYPES:
+            self._dump_primitive(obj, obj_class, variant, config.max_string)
+        elif obj_class in LIST_TYPE:
             self._dump_list(obj, variant, current_depth, config, log_errors)
-        elif isinstance(obj, dict):
+        elif obj_class in DICT_TYPE:
             self._dump_dictionary(obj, variant, current_depth, config, log_errors)
-        elif isinstance(obj, BaseException):
+        elif BaseException in obj_mro:
             self._dump_exception(obj, variant, current_depth, config, log_errors)
-        elif isinstance(obj, TracebackType):
+        elif obj_class is TracebackType:
             self._dump_traceback(obj, variant, current_depth, config, log_errors)
-        elif NamespaceSerializer2.is_numpy_obj(obj):
-            self._dump_primitive(obj.item(), original_type, variant, config.max_string)
-        elif NamespaceSerializer2.is_torch_obj(obj):
-            self._dump_primitive(str(obj), original_type, variant, config.max_string)
-        elif NamespaceSerializer2.is_multidict_obj(obj):
-            self._dump_primitive(str(obj), original_type, variant, config.max_string)
-        elif NamespaceSerializer2.is_protobuf_obj(obj):
+        elif NamespaceSerializer2.is_numpy_obj(obj_mro):
+            item = obj.item()
+            self._dump_primitive(item, type(item), variant, config.max_string)
+        elif NamespaceSerializer2.is_torch_obj(obj_class):
+            self._dump_primitive(str(obj), str, variant, config.max_string)
+        elif NamespaceSerializer2.is_multidict_obj(obj_class):
+            self._dump_primitive(str(obj), str, variant, config.max_string)
+        elif NamespaceSerializer2.is_protobuf_obj(obj_class):
             self._dump_protobuf(obj, variant, current_depth, config, log_errors)
-        elif NamespaceSerializer2.is_bson(obj):
-            self._dump_bson(obj, variant, current_depth, config, log_errors)
-        elif hasattr(obj, '__dict__') or hasattr(obj, '__slots__'):
-            self._dump_user_class(obj, variant, current_depth, config, log_errors)
+        elif NamespaceSerializer2.is_bson(obj_class):
+            self._dump_bson(obj, obj_class, variant, current_depth, config, log_errors)
         else:
-            self._dump_not_supported(obj, variant)
+            self._dump_user_class(obj, obj_class, variant, current_depth, config, log_errors)
 
     def _dump_traceback(self, obj, variant, current_depth, config, log_errors):
         # python separates the "forward" stack (callees of the except clause)
@@ -141,31 +141,31 @@ class NamespaceSerializer2(NamespaceSerializerBase):
         variant.bytes_index_in_cache = self._get_string_index_in_cache(value)
         self.estimated_pending_bytes += 6  # Header + number + header + number
 
-    def _dump_primitive(self, obj, original_type, variant, max_string):
+    def _dump_primitive(self, obj, obj_class, variant, max_string):
         if obj is None:
             self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_NONE)
 
-        elif isinstance(obj, six.integer_types) and (obj < Int64ValueChecker._MAX) and (obj > Int64ValueChecker._MIN):
-            self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_LONG)
-            variant.long_value = int(obj)
-            self.estimated_pending_bytes += 3  # Header + number
+        elif obj_class in six.integer_types:
+            if (obj < Int64ValueChecker._MAX) and (obj > Int64ValueChecker._MIN):
+                self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_LONG)
+                variant.long_value = int(obj)
+                self.estimated_pending_bytes += 3  # Header + number
+            else:
+                self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_LARGE_INT)
+                variant.bytes_index_in_cache = self._get_string_index_in_cache(str(obj))
+                self.estimated_pending_bytes += 3  # Header + number
 
-        elif isinstance(obj, six.integer_types):
-            self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_LARGE_INT)
-            variant.bytes_index_in_cache = self._get_string_index_in_cache(str(obj))
-            self.estimated_pending_bytes += 3  # Header + number
-
-        elif isinstance(obj, bool):
+        elif obj_class is bool:
             self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_LONG)
             variant.long_value = int(obj)
             self.estimated_pending_bytes += 2  # Header + short number
 
-        elif isinstance(obj, float):
+        elif obj_class is float:
             self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_DOUBLE)
             variant.double_value = float(obj)
             self.estimated_pending_bytes += 7  # Header + 64 bit float
 
-        elif isinstance(obj, decimal.Decimal):
+        elif obj_class is decimal.Decimal:
             serialized_decimal = str(obj)
 
             self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_STRING)
@@ -173,7 +173,7 @@ class NamespaceSerializer2(NamespaceSerializerBase):
             variant.bytes_index_in_cache = self._get_string_index_in_cache(str(serialized_decimal))
             self.estimated_pending_bytes += 6  # Header + number + header + number
 
-        elif isinstance(obj, six.string_types):
+        elif obj_class in self.STRING_TYPES:
             self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_STRING)
             variant.original_size = len(obj)
 
@@ -184,7 +184,7 @@ class NamespaceSerializer2(NamespaceSerializerBase):
             variant.bytes_index_in_cache = self._get_string_index_in_cache(string)
             self.estimated_pending_bytes += 6  # Header + number + header + number
 
-        elif isinstance(obj, self.BINARY_TYPES) or original_type == 'binary_type':
+        elif obj_class in self.BINARY_TYPES or obj_class.__name__ == 'binary_type':
             self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_BINARY)
             variant.original_size = len(obj)
 
@@ -194,25 +194,27 @@ class NamespaceSerializer2(NamespaceSerializerBase):
             variant.bytes_index_in_cache = self._get_bytes_index_in_cache(bytes(obj))
             self.estimated_pending_bytes += 6  # Header + number + header + number
 
-        elif isinstance(obj, self.CODE_TYPES):
+        elif obj_class in self.CODE_TYPES:
             self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_CODE_OBJECT)
             code_value = variant.code_values.add()
             code_value.name_index_in_cache = self._get_string_index_in_cache(self.normalize_string(obj.__name__))
             if hasattr(obj, '__code__') and hasattr(obj.__code__, 'co_filename'):
-                code_value.filename_index_in_cache = self._get_string_index_in_cache(self.normalize_string(obj.__code__.co_filename))
+                code_value.filename_index_in_cache = self._get_string_index_in_cache(
+                    self.normalize_string(obj.__code__.co_filename))
                 code_value.lineno = int(obj.__code__.co_firstlineno)
             if hasattr(obj, '__module__') and obj.__module__:
-                code_value.module_index_in_cache = self._get_string_index_in_cache(self.normalize_string(obj.__module__))
+                code_value.module_index_in_cache = self._get_string_index_in_cache(
+                    self.normalize_string(obj.__module__))
 
             self.estimated_pending_bytes += 14  # Header + size + (header + number) * 4
 
-        elif isinstance(obj, complex):
+        elif obj_class is complex:
             self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_COMPLEX)
             variant.complex_value.real = float(obj.real)
             variant.complex_value.imaginary = float(obj.imag)
             self.estimated_pending_bytes += 8  # Large header + size + (header + 64 bit float) * 2
 
-        elif isinstance(obj, datetime.datetime):
+        elif obj_class is datetime.datetime:
             self._dump_datetime(obj, variant)
 
         else:
@@ -229,7 +231,7 @@ class NamespaceSerializer2(NamespaceSerializerBase):
     def _dump_list(self, collection, variant, current_depth, config, log_errors):
         self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_LIST)
 
-        if ListNamespace.is_numpy_obj(collection):
+        if self.is_numpy_array(type(collection)):
             collection = collection.tolist()
             if not collection:
                 collection = []
@@ -272,7 +274,7 @@ class NamespaceSerializer2(NamespaceSerializerBase):
 
     def _dump_protobuf(self, obj, variant, current_depth, config, log_errors):
         self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_OBJECT)
-        if hasattr(obj, 'DESCRIPTOR'):
+        if self.has_attr_safe(obj, 'DESCRIPTOR'):
             for field in obj.ListFields():
                 try:
                     variant.attribute_names_in_cache.append(self._get_string_index_in_cache(field[0].name))
@@ -284,11 +286,11 @@ class NamespaceSerializer2(NamespaceSerializerBase):
                 except Exception:  # for now we just ignore errors when dumping protobuf
                     pass
 
-    def _dump_bson(self, obj, variant, current_depth, config, log_errors):
-        if isinstance(obj, ObjectId):
-            return self._dump_primitive(str(obj), str(type(obj)), variant, config.max_string)
+    def _dump_bson(self, obj, obj_class, variant, current_depth, config, log_errors):
+        if obj_class is ObjectId:
+            return self._dump_primitive(str(obj), str, variant, config.max_string)
 
-        if isinstance(obj, RawBSONDocument):
+        if obj_class is RawBSONDocument:
             return self._dump_dictionary(decode(obj.raw), variant, current_depth, config, log_errors)
 
         return None
@@ -303,13 +305,15 @@ class NamespaceSerializer2(NamespaceSerializerBase):
 
             self._dump_python_object(exc.args, args_variant, current_depth + 1, config, log_errors)
 
-    def _dump_user_class(self, obj, variant, current_depth, config, log_errors):
+    def _dump_user_class(self, obj, obj_class, variant, current_depth, config, log_errors):
         object_weight = current_depth + 1
 
         self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_OBJECT)
 
-        if getattr(obj, '__dict__', None) is not None:
-            for key, value in six.iteritems(obj.__dict__.copy()):
+        dict_items = self.get_attr_safe(obj, '__dict__')
+        if dict_items is not None:
+            dict_items = dict_items.copy()
+            for key, value in six.iteritems(dict_items):
                 if key not in self.BUILTIN_ATTRIBUTES_IGNORE:
                     if object_weight >= config.max_depth:
                         self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_OBJECT, True)
@@ -321,13 +325,18 @@ class NamespaceSerializer2(NamespaceSerializerBase):
 
                     self._dump_python_object(value, attribute_value_variant, object_weight, config, log_errors)
 
-        if getattr(obj, '__slots__', None) is not None:
-            items = obj.__slots__
+        slots = self.get_attr_safe(obj, '__slots__')
+        if slots is None:
+            full_class_path = '{0}.{1}'.format(obj_class.__module__, obj_class.__name__)
+            if self.is_in_get_attr_allowlist(full_class_path):
+                slots = getattr(obj, '__slots__', None)
+        if slots is not None:
             # py4j (used by pyspark to communicate with Java proxy objects) sets __slots__ to Java proxy objects,
-            # and supports __dir__ instead.
-            if not isinstance(items, Iterable):
-                items = dir(items)
-            for key in list(items):
+            # and supports __dir__ instead
+            # Also, safe to use isinstance on slots object
+            if not isinstance(slots, Iterable):
+                slots = dir(slots)
+            for key in list(slots):
                 if key not in self.BUILTIN_ATTRIBUTES_IGNORE:
                     if object_weight >= config.max_depth:
                         self.dump_variant_type(variant, variant_pb2.Variant.VARIANT_OBJECT, True)
@@ -338,6 +347,7 @@ class NamespaceSerializer2(NamespaceSerializerBase):
                     self.estimated_pending_bytes += 6  # Header + number + header + size
 
                     try:
+                        # Once we verified it is using slots, it's safe to go through getattr
                         value = getattr(obj, key)
                     except AttributeError:
                         value = None
