@@ -1,22 +1,13 @@
 import six
-import datetime
 
 from .namespace import Namespace
 
 from rook.exceptions import RookKeyNotFound, RookAttributeNotFound, RookInvalidMethodArguments
-
-try:
-    from bson import ObjectId
-except ImportError:
-    pass # In case pymongo isn't loaded.
+from ..namespace_serializer_base import NamespaceSerializerBase
+from .variable_types import LIST_TYPE, DICT_TYPE
 
 
 class PythonObjectNamespace(Namespace):
-    try:
-        BINARY_TYPES = (buffer, bytearray)
-    except NameError:
-        BINARY_TYPES = (bytearray, bytes)
-
     class ObjectDumpConfig(object):
 
         STRICT_MAX_DEPTH = 2
@@ -74,9 +65,10 @@ class PythonObjectNamespace(Namespace):
 
         @classmethod
         def tailor_limits(cls, obj):
-            if isinstance(obj, (six.string_types,bytes)):
+            obj_class = type(obj)
+            if obj_class in NamespaceSerializerBase.STRING_TYPES or obj_class in NamespaceSerializerBase.BINARY_TYPES:
                 return PythonObjectNamespace.ObjectDumpConfig(1, 0, 0, cls.UNLIMITED_STRING)
-            if isinstance(obj, (list, dict, set, tuple)) and len(obj) > cls.TOLERANT_MAX_WIDTH:
+            if (obj_class in LIST_TYPE or obj_class in DICT_TYPE) and len(obj) > cls.TOLERANT_MAX_WIDTH:
                 return PythonObjectNamespace.ObjectDumpConfig(max_width=cls.UNLIMITED_COLLECTION_WIDTH)
             else:
                 return PythonObjectNamespace.ObjectDumpConfig.tolerant_limits(obj)
@@ -84,58 +76,43 @@ class PythonObjectNamespace(Namespace):
     def __init__(self, obj, dump_config=None, methods=()):
         super(PythonObjectNamespace, self).__init__(methods + self.METHODS)
         self.obj = obj
+        self.obj_class = type(obj)
         self.dump_config = dump_config or self.ObjectDumpConfig()
 
-        try:
-            if ObjectId and isinstance(obj, ObjectId):
-                self.obj = str(obj)
-                self.type = str(str)
-        except Exception:
-            # In case we have the case of BadObject, that throws an exception when you attempt to access any attribute.
-            pass
-    #
-    @classmethod
-    def get_common_type(cls, obj):
-        # NOTE: This list is partially duplicated with the list in NamespaceSerializer._load_variant
-        if obj is None:
-            return u'null'
-        elif isinstance(obj, six.integer_types):
-            return u'int'
-        elif isinstance(obj, six.string_types):
-            return u'string'
-        elif isinstance(obj, float):
-            return u'float'
-        elif isinstance(obj, cls.BINARY_TYPES):
-            return u'buffer'
-        elif isinstance(obj, datetime.datetime):
-            return u'datetime'
-        elif isinstance(obj, (list, set, tuple)):
-            from .collection_namespace import ListNamespace
-            return ListNamespace.get_common_type(obj)
-        elif isinstance(obj, (dict)):
-            return u'dict'
-        elif isinstance(obj, complex):
-            return u'complex'
-        else:
-            raise ValueError("Unknown type- %s", str(type(obj)))
+    if six.PY3:
+        def _read_attr_unsafe(self, name):
+            return object.__getattribute__(self.obj, name)
+    else:
+        def _read_attr_unsafe(self, name):
+            # In python 2, you don't technically have to inherit from `object`, so some methods aren't using
+            # `__getattribute__` like object does. Therefore, we must use the unsafe alternatives
+            return getattr(self.obj, name)
 
     def read_attribute(self, name):
         try:
-            if hasattr(self.obj, '__dict__') and self.obj.__dict__ and name in self.obj.__dict__:
-                return PythonObjectNamespace(self.obj.__dict__[name])
-            if hasattr(self.obj, '__slots__') and self.obj.__slots__ and name in self.obj.__slots__:
-                return PythonObjectNamespace(getattr(self.obj, name))
-
-            raise RookAttributeNotFound(name)
+            obj_dict = self._read_attr_unsafe('__dict__')
+            if obj_dict is not None and name in obj_dict:
+                return PythonObjectNamespace(obj_dict[name])
         except AttributeError:
-            raise RookAttributeNotFound(name)
+            pass  # Try using __slots__
+
+        try:
+            obj_slots = self._read_attr_unsafe('__slots__')
+            if obj_slots is not None and name in obj_slots:
+                # Once we verified it is using slots, it's safe to go through getattr
+                return PythonObjectNamespace(getattr(self.obj, name))
+        except AttributeError:
+            pass  # Go through to the end
+
+        raise RookAttributeNotFound(name)
 
     def read_key(self, key):
         try:
             return PythonObjectNamespace(self.obj[key])
         except (KeyError, IndexError, TypeError):
             # If key is a string and object is a dictionary check if any key within the dictionary stringifies to key
-            if isinstance(key, six.string_types) and isinstance(self.obj, dict):
+            key_class = type(key)
+            if key_class in six.string_types and self.obj_class in DICT_TYPE:
                 for iteratorKey in self.obj:
                     if str(iteratorKey) == key:  # The str() is the important part
                         return PythonObjectNamespace(self.obj[iteratorKey])
@@ -143,10 +120,7 @@ class PythonObjectNamespace(Namespace):
             raise RookKeyNotFound(key)
 
     def type(self, args=None):
-        return PythonObjectNamespace(self.serialize_type())
-
-    def serialize_type(self):
-        return str(type(self.obj))
+        return PythonObjectNamespace(str(self.obj_class))
 
     def size(self, args=None):
         return PythonObjectNamespace(len(self.obj))
