@@ -17,7 +17,10 @@
 // Ensure that Python.h is included before any other header.
 #include "common.h"
 
+#include "opcode_caches.h"
 #include "bytecode_manipulator.h"
+#include "linetable/linetable.h"
+#include <algorithm>
 
 namespace devtools {
 namespace cdbg {
@@ -27,6 +30,7 @@ namespace cdbg {
 enum PythonOpcodeType {
   SEQUENTIAL_OPCODE,
   BRANCH_DELTA_OPCODE,
+  BRANCH_BACKWARD_DELTA_OPCODE,
   BRANCH_ABSOLUTE_OPCODE,
   YIELD_OPCODE
 };
@@ -116,27 +120,53 @@ static int GetInstructionsSize(
 // Classification of an opcode.
 static PythonOpcodeType GetOpcodeType(uint8 opcode) {
   switch (opcode) {
+      // Yield opcodes
     case YIELD_VALUE:
-#if PY_MAJOR_VERSION >= 3
+#if PY_MAJOR_VERSION >= 3 && PY_VERSION_HEX < 0x030B0000
     case YIELD_FROM:
 #endif
       return YIELD_OPCODE;
 
+      // Relative jump opcodes
     case FOR_ITER:
     case JUMP_FORWARD:
+#if PY_VERSION_HEX >= 0x030B0000
+    case JUMP_IF_FALSE_OR_POP:
+    case JUMP_IF_TRUE_OR_POP:
+    case POP_JUMP_FORWARD_IF_FALSE:
+    case POP_JUMP_FORWARD_IF_TRUE:
+    case SEND:
+    case POP_JUMP_FORWARD_IF_NOT_NONE:
+    case POP_JUMP_FORWARD_IF_NONE:
+#endif
+
 #if PY_VERSION_HEX < 0x03080000
     // Removed in Python 3.8.
     case SETUP_LOOP:
     case SETUP_EXCEPT:
 #endif
+#if PY_VERSION_HEX < 0x030B0000
     case SETUP_FINALLY:
     case SETUP_WITH:
+#endif
 #if PY_VERSION_HEX >= 0x03080000 && PY_VERSION_HEX < 0x03090000
     // Added in Python 3.8 and removed in 3.9
     case CALL_FINALLY:
 #endif
       return BRANCH_DELTA_OPCODE;
 
+#if PY_VERSION_HEX >= 0x030B0000
+    case JUMP_BACKWARD_NO_INTERRUPT:
+    case JUMP_BACKWARD:
+    case POP_JUMP_BACKWARD_IF_NOT_NONE:
+    case POP_JUMP_BACKWARD_IF_NONE:
+    case POP_JUMP_BACKWARD_IF_FALSE:
+    case POP_JUMP_BACKWARD_IF_TRUE:
+        return BRANCH_BACKWARD_DELTA_OPCODE;
+#endif
+
+#if PY_VERSION_HEX < 0x030B0000
+      // Absolute jump opcodes
     case JUMP_IF_FALSE_OR_POP:
     case JUMP_IF_TRUE_OR_POP:
     case JUMP_ABSOLUTE:
@@ -147,6 +177,7 @@ static PythonOpcodeType GetOpcodeType(uint8 opcode) {
     case CONTINUE_LOOP:
 #endif
       return BRANCH_ABSOLUTE_OPCODE;
+#endif
 
     default:
       return SEQUENTIAL_OPCODE;
@@ -199,7 +230,7 @@ static PythonInstruction ReadInstruction(
 
 #if PY_VERSION_HEX >= 0x03060000
   if (bytecode.end() - it < 2) {
-    LOG(ERROR) << "Buffer underflow";
+    DEBUG("Buffer underflow");
     return kInvalidInstruction;
   }
 
@@ -208,7 +239,7 @@ static PythonInstruction ReadInstruction(
     it += 2;
     instruction.size += 2;
     if (bytecode.end() - it < 2) {
-      LOG(ERROR) << "Buffer underflow";
+      DEBUG("Buffer underflow");
       return kInvalidInstruction;
     }
   }
@@ -316,9 +347,22 @@ static void WriteInstructions(
 // method is assumed to be defined in the specified item of a constants tuple.
 static std::vector<PythonInstruction> BuildMethodCall(int const_index) {
   std::vector<PythonInstruction> instructions;
+#if PY_VERSION_HEX >= 0x030B0000
+  instructions.push_back(PythonInstructionArg(PUSH_NULL, const_index));
+  instructions.push_back(PythonInstructionArg(LOAD_CONST, const_index));
+  instructions.push_back(PythonInstructionArg(PRECALL, 0));
+  instructions.push_back(PythonInstructionArg(CACHE, 0));
+  instructions.push_back(PythonInstructionArg(CALL, 0));
+  instructions.push_back(PythonInstructionArg(CACHE, 0));
+  instructions.push_back(PythonInstructionArg(CACHE, 0));
+  instructions.push_back(PythonInstructionArg(CACHE, 0));
+  instructions.push_back(PythonInstructionArg(CACHE, 0));
+  instructions.push_back(PythonInstructionNoArg(POP_TOP));
+#else
   instructions.push_back(PythonInstructionArg(LOAD_CONST, const_index));
   instructions.push_back(PythonInstructionArg(CALL_FUNCTION, 0));
   instructions.push_back(PythonInstructionNoArg(POP_TOP));
+#endif
 
   return instructions;
 }
@@ -327,10 +371,16 @@ static std::vector<PythonInstruction> BuildMethodCall(int const_index) {
 BytecodeManipulator::BytecodeManipulator(
     std::vector<uint8> bytecode,
     const bool has_lnotab,
-    std::vector<uint8> lnotab)
-    : has_lnotab_(has_lnotab) {
+    std::vector<uint8> lnotab,
+    const bool has_exception_table,
+    std::vector<uint8> exception_table,
+    PyCodeObject *code_object)
+    : has_lnotab_(has_lnotab),
+      has_exception_table_(has_exception_table) {
   data_.bytecode = std::move(bytecode);
   data_.lnotab = std::move(lnotab);
+  data_.exception_table = std::move(exception_table);
+  data_.code_object = code_object;
 
   strategy_ = STRATEGY_INSERT;  // Default strategy.
   for (auto it = data_.bytecode.begin(); it < data_.bytecode.end(); ) {
@@ -356,13 +406,17 @@ bool BytecodeManipulator::InjectMethodCall(
   Data new_data = data_;
   switch (strategy_) {
     case STRATEGY_INSERT:
+      DDEBUG("BytecodeManipulator::InjectMethodCall: InsertMethodCall\n");
       if (!InsertMethodCall(&new_data, offset, callable_const_index)) {
-        return false;
+          DEBUG("InsertMethodCall failed!!\n");
+          return false;
       }
       break;
 
     case STRATEGY_APPEND:
+      DDEBUG("BytecodeManipulator::InjectMethodCall: Append method call\n");
       if (!AppendMethodCall(&new_data, offset, callable_const_index)) {
+        DEBUG("AppendMethodCall failed!!\n");
         return false;
       }
       break;
@@ -405,39 +459,95 @@ struct Insertion {
 // InsertAndUpdateBranchInstructions.
 static const int kMaxInsertionIterations = 10;
 
-
-// Updates the line number table for an insertion in the bytecode.
-// This is different than what the Python 2 version of InsertMethodCall() does.
-// It should be more accurate, but is confined to Python 3 only for safety.
-// This handles the case of adding insertion for EXTENDED_ARG better.
-// Example for inserting 2 bytes at offset 2:
-// lnotab: [{2, 1}, {4, 1}] // {offset_delta, line_delta}
-// Old algorithm: [{2, 0}, {2, 1}, {4, 1}]
-// New algorithm: [{2, 1}, {6, 1}]
-// In the old version, trying to get the offset to insert a breakpoint right
-// before line 1 would result in an offset of 2, which is inaccurate as the
-// instruction before is an EXTENDED_ARG which will now be applied to the first
-// instruction inserted instead of its original target.
-static void InsertAndUpdateLnotab(int offset, int size,
-                                  std::vector<uint8>* lnotab) {
-  int current_offset = 0;
-  for (auto it = lnotab->begin(); it != lnotab->end(); it += 2) {
-    current_offset += it[0];
-
-    if (current_offset > offset) {
-      int remaining_size = it[0] + size;
-      int remaining_lines = it[1];
-      it = lnotab->erase(it, it + 2);
-      while (remaining_size > 0xFF) {
-        it = lnotab->insert(it, 0xFF) + 1;
-        it = lnotab->insert(it, 0) + 1;
-        remaining_size -= 0xFF;
-      }
-      it = lnotab->insert(it, remaining_size) + 1;
-      it = lnotab->insert(it, remaining_lines) + 1;
-      return;
+static int parse_varint(std::vector<uint8>::iterator *it) {
+    int num = 0;
+    uint8 bitmask = (1 << 6) - 1; // = 63
+    uint8 byte = **it;
+    (*it)++;
+    num = byte & bitmask;
+    while (byte & (1 << 6)) {
+        num <<= 6;
+        byte = **it;
+        (*it)++;
+        num |= byte & bitmask;
     }
-  }
+
+    return num;
+}
+
+
+static std::vector<uint8> write_varint(int val, bool is_entry_start) {
+    std::vector<uint8> res;
+    uint8 flags = 0;
+    uint8 bitmask = (1 << 6) - 1; // = 63
+
+    while (val >= 64) {
+        res.push_back(val & bitmask);
+        val >>= 6;
+    }
+    res.push_back(val);
+
+    std::reverse(res.begin(), res.end());
+
+    // If it is the start of an entry, we need to set the "entry-start" flag on
+    if (is_entry_start) {
+        flags |= (1 << 7);
+    }
+
+    // Set the "more-data" flag for every byte but the last
+    for (auto it = res.begin(); it != res.end() - 1; it++) {
+        flags |= (1 << 6);
+        *it |= flags;
+        flags = 0;
+    }
+
+    return res;
+}
+
+static void InsertAndUpdateExceptionTable(int offset, int size, std::vector<uint8> *exception_table) {
+    int start = 0;
+    int length = 0;
+    int end = 0;
+    int target = 0;
+    int dl = 0;
+
+    DEBUG("Creating new exception table, with bp inserted at %d (size = %d)\n", offset, size);
+
+    std::vector<uint8> new_exception_table;
+    std::vector<uint8> varint_val;
+
+    for (auto it = exception_table->begin(); it != exception_table->end();) {
+        // Parse the entry
+        start = parse_varint(&it) * 2;
+        length = parse_varint(&it) * 2;
+        end = start + length - 2;
+        target = parse_varint(&it) * 2;
+        dl = parse_varint(&it);
+
+        if (start >= offset) {
+            start += size;
+        }
+        if (end >= offset) {
+            end += size;
+        }
+        if (target >= offset) {
+            target += size;
+        }
+
+        DDEBUG("start: %d, end: %d, target: %d, dl: %d\n", start, end, target, dl);
+
+        // Append the new entry
+        varint_val = write_varint(start / 2, true);
+        new_exception_table.insert(new_exception_table.end(), varint_val.begin(), varint_val.end());
+        varint_val = write_varint((end - start + 2) / 2, false);
+        new_exception_table.insert(new_exception_table.end(), varint_val.begin(), varint_val.end());
+        varint_val = write_varint(target / 2, false);
+        new_exception_table.insert(new_exception_table.end(), varint_val.begin(), varint_val.end());
+        varint_val = write_varint(dl, false);
+        new_exception_table.insert(new_exception_table.end(), varint_val.begin(), varint_val.end());
+    }
+
+    *exception_table = new_exception_table;
 }
 
 
@@ -522,6 +632,11 @@ static bool InsertAndUpdateBranchInstructions(
         int32 target = it->current_offset + inst_size + arg;
         need_to_update = it->current_offset < insertion.current_offset &&
                          insertion.current_offset < target;
+      } else if (opcode_type == BRANCH_BACKWARD_DELTA_OPCODE) {
+          int inst_size = std::max(instruction.size, it->original_size);
+          int32 target = it->current_offset + inst_size - arg;
+          need_to_update = it->current_offset > insertion.current_offset &&
+                           insertion.current_offset > target;
       } else if (opcode_type == BRANCH_ABSOLUTE_OPCODE) {
         // For absolute branches, the argument needs to be updated if the
         // insertion before the target.
@@ -586,6 +701,7 @@ bool BytecodeManipulator::InsertMethodCall(
 
     PythonOpcodeType opcode_type = GetOpcodeType(instruction.opcode);
     if (opcode_type == BRANCH_DELTA_OPCODE ||
+        opcode_type == BRANCH_BACKWARD_DELTA_OPCODE ||
         opcode_type == BRANCH_ABSOLUTE_OPCODE) {
       updated_instructions.push_back(
           UpdatedInstruction { instruction, instruction.size, current_offset });
@@ -595,7 +711,7 @@ bool BytecodeManipulator::InsertMethodCall(
   }
 
   if (!offset_valid) {
-    LOG(ERROR) << "Offset " << offset << " is mid instruction or out of range";
+      DEBUG("Offset %d is mid instruction or out of range\n", offset);
     return false;
   }
 
@@ -605,7 +721,7 @@ bool BytecodeManipulator::InsertMethodCall(
   int method_call_size = GetInstructionsSize(method_call_instructions);
   if (!InsertAndUpdateBranchInstructions({ method_call_size, offset },
                                          updated_instructions)) {
-    LOG(ERROR) << "Too many instruction argument upgrades required";
+    DEBUG("Too many instruction argument upgrades required");
     return false;
   }
 
@@ -613,7 +729,10 @@ bool BytecodeManipulator::InsertMethodCall(
   data->bytecode.insert(data->bytecode.begin() + offset, method_call_size, NOP);
   WriteInstructions(data->bytecode.begin() + offset, method_call_instructions);
   if (has_lnotab_) {
-    InsertAndUpdateLnotab(offset, method_call_size, &data->lnotab);
+      linetable::InsertAndUpdateLinetable(offset, method_call_size, &data->lnotab, data->code_object);
+  }
+  if (has_exception_table_) {
+      InsertAndUpdateExceptionTable(offset, method_call_size, &data->exception_table);
   }
 
   // Write new branch instructions.
@@ -626,7 +745,10 @@ bool BytecodeManipulator::InsertMethodCall(
     if (size_diff > 0) {
       data->bytecode.insert(data->bytecode.begin() + offset, size_diff, NOP);
       if (has_lnotab_) {
-        InsertAndUpdateLnotab(it->current_offset, size_diff, &data->lnotab);
+          linetable::InsertAndUpdateLinetable(offset, method_call_size, &data->lnotab, data->code_object);
+      }
+      if (has_exception_table_) {
+          InsertAndUpdateExceptionTable(it->current_offset, size_diff, &data->exception_table);
       }
     } else if (size_diff < 0) {
       // The Python compiler sometimes prematurely adds EXTENDED_ARG with an
@@ -757,7 +879,12 @@ bool BytecodeManipulator::AppendMethodCall(
     int offset,
     int const_index) const {
 
-#if PY_VERSION_HEX >= 0x030A0000
+#if PY_VERSION_HEX >= 0x030B0000
+    int jump_forward_delta = data->bytecode.size() - (offset + 2);
+    jump_forward_delta /= 2;
+    PythonInstruction trampoline =
+            PythonInstructionArg(JUMP_FORWARD, jump_forward_delta);
+#elif PY_VERSION_HEX >= 0x030A0000
     PythonInstruction trampoline =
       PythonInstructionArg(JUMP_ABSOLUTE, (data->bytecode.size())/2);
 #else
@@ -770,7 +897,7 @@ bool BytecodeManipulator::AppendMethodCall(
   for (auto it = data->bytecode.begin() + offset;
       relocated_size < trampoline.size; ) {
     if (it >= data->bytecode.end()) {
-      LOG(ERROR) << "Not enough instructions";
+        DEBUG("Not enough instructions");
       return false;
     }
 
@@ -791,15 +918,28 @@ bool BytecodeManipulator::AppendMethodCall(
     //    object in between "yield" calls. If the breakpoint is added or
     //    removed, subsequent calls into the generator will jump into invalid
     //    location.
+
+    // TODO: Allow relative jumps, since we have JUMP_BACKWARD now...
     if ((opcode_type == BRANCH_DELTA_OPCODE) ||
+        (opcode_type == BRANCH_BACKWARD_DELTA_OPCODE) ||
         (opcode_type == YIELD_OPCODE)) {
-      LOG(ERROR) << "Not enough space for trampoline";
+      DEBUG("Not enough space for trampoline");
       return false;
     }
 
     relocated_instructions.push_back(instruction);
     relocated_size += instruction.size;
     it += instruction.size;
+
+    // Add cache opcodes
+#if PY_VERSION_HEX >= 0x030B0000
+    for (int i = 1; i <= GetCacheCount(instruction.opcode); i++) {
+        PythonInstruction cache_inst = PythonInstructionArg(CACHE, 0);
+        relocated_instructions.push_back(cache_inst);
+        relocated_size += cache_inst.size;
+        it += cache_inst.size;
+    }
+#endif
   }
 
   for (auto it = data->bytecode.begin(); it < data->bytecode.end(); ) {
@@ -810,6 +950,7 @@ bool BytecodeManipulator::AppendMethodCall(
 
     const PythonOpcodeType opcode_type = GetOpcodeType(instruction.opcode);
     if ((opcode_type == BRANCH_DELTA_OPCODE) ||
+        (opcode_type == BRANCH_BACKWARD_DELTA_OPCODE) ||
         (opcode_type == BRANCH_ABSOLUTE_OPCODE)) {
       const int branch_target =
           GetBranchTarget(it - data->bytecode.begin(), instruction);
@@ -839,7 +980,7 @@ bool BytecodeManipulator::AppendMethodCall(
       // this issue is not common enough.
       if ((branch_target > offset) &&
           (branch_target < offset + relocated_size)) {
-        LOG(ERROR) << "Jump into relocated instruction detected";
+        DEBUG("Jump into relocated instruction detected");
         return false;
       }
     }
@@ -848,11 +989,34 @@ bool BytecodeManipulator::AppendMethodCall(
   }
 
   std::vector<PythonInstruction> appendix = BuildMethodCall(const_index);
+  int method_call_size = GetInstructionsSize(appendix);
   appendix.insert(
       appendix.end(),
       relocated_instructions.begin(),
       relocated_instructions.end());
-#if PY_VERSION_HEX >= 0x030A0000
+#if PY_VERSION_HEX >= 0x030B0000
+  // We don't know how many opcodes the actual JUMP_BACKWARD would take (because of EXTENDED_ARG),
+  // so we iterate over the possible sizes, and check if we guessed correctly.
+  int jump_offset = data->bytecode.size() + method_call_size + trampoline.size;
+  int dst_offset = offset + trampoline.size;
+  int pre_jump_delta = (jump_offset - dst_offset) / 2;
+  bool is_added = false;
+  for (int i = 1; i <= 4; i++) {
+      int delta = pre_jump_delta + i;
+      PythonInstruction jump = PythonInstructionArg(JUMP_BACKWARD, delta);
+      if (jump.size / 2 == i) {
+          appendix.push_back(jump);
+          is_added = true;
+          break;
+      }
+  }
+
+
+  // Shouldn't happen, but we check that we actually added the jump opcode here
+  if (!is_added) {
+      return false;
+  }
+#elif PY_VERSION_HEX >= 0x030A0000
   appendix.push_back(PythonInstructionArg(
       JUMP_ABSOLUTE,
       (offset + relocated_size)/2));
