@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 import os
+import sys
 import types
 
 import six
@@ -32,6 +33,8 @@ class Bdb(object):
     def __init__(self):
         self.fncache = {}
         self._cookies = {}
+        self._cookies_to_func = {}
+        self._func_to_original_co_code = {}
         self._bp_status = {}
         self.user_line = None
         self.rookout_dev_debug = os.environ.get('ROOKOUT_DEV_DEBUG')
@@ -72,10 +75,11 @@ class Bdb(object):
             if hasattr(module, '__file__'):
                 logger.debug("CodeNotFound module filename %s", module.__file__)
 
-            for cobj in _GetModuleCodeObjects(module):
-                logger.debug("Name: %s", cobj.co_name)
-                for cline in _GetLineNumbers(cobj):
-                    logger.debug("Name: %s, Line %d", cobj.co_name, cline)
+            for code_object_node in _GetModuleCodeObjects(module):
+                inner_cobj = code_object_node.obj
+                logger.debug("Name: %s", inner_cobj.co_name)
+                for cline in _GetLineNumbers(inner_cobj):
+                    logger.debug("Name: %s, Line %d", inner_cobj.co_name, cline)
 
             if code_object == (None, None):
                 raise RookBdbCodeNotFound(filename=filename)
@@ -84,7 +88,24 @@ class Bdb(object):
 
         self._set_break_code_object(code_object, filename, lineno, aug_id)
 
-    def _set_break_code_object(self, code_object, filename, lineno, aug_id):
+    def _set_code_object(self, code_object_node, code_object):
+        if isinstance(code_object_node.parent, tuple):
+            co_consts = list(code_object_node.parent)
+            index = co_consts.index(code_object_node.obj)
+            co_consts[index] = code_object
+            # In this case, the parent of obj is the tuple itself, and the grandparent
+            # is the one holding the tuple in co_consts
+            cdbg_native.UpdateCoConsts(code_object_node.grandparent, tuple(co_consts))
+        else:
+            logger.debug("Trying to change __code__ of object of type %s", str(type(code_object_node.parent)))
+            code_object_node.parent.__code__ = code_object
+
+    def _set_break_code_object(self, code_object_node, filename, lineno, aug_id):
+        try:
+            code_object = self._func_to_original_co_code[code_object_node.get_id()]
+        except KeyError:
+            code_object = code_object_node.obj
+
         # Install the breakpoint
         bp_status = BPStatus()
         callback_partial = self._get_callback_partial(callback, lineno, code_object, bp_status, filename, aug_id)
@@ -98,8 +119,14 @@ class Bdb(object):
                 dis.dis(code_object, file=f)
                 dis_output = f.getvalue()
                 logger.debug("code_object before patch aug_id: %s, filename: %s, lineno: %d, dis: %s:", aug_id, filename, lineno, dis_output)
-        cookie = cdbg_native.SetConditionalBreakpoint(code_object, lineno, None, callback_partial,
-                                                      error_callback_partial)
+        if sys.version_info >= (3, 11, 0):
+            cookie, new_co_code = cdbg_native.SetConditionalBreakpoint(code_object, lineno, None, callback_partial,
+                                                                       error_callback_partial)
+            new_code_object = code_object.replace(co_code=new_co_code)
+            self._set_code_object(code_object_node, new_code_object)
+        else:
+            cookie = cdbg_native.SetConditionalBreakpoint(code_object, lineno, None, callback_partial,
+                                                          error_callback_partial)
 
         if self.rookout_dev_debug:
             with io.StringIO() as f:
@@ -111,6 +138,9 @@ class Bdb(object):
             raise RookBdbSetBreakpointFailed("%s on line %d" % (code_object.co_name, lineno))
 
         self._cookies[aug_id] = cookie
+        self._cookies_to_func[cookie] = code_object_node
+        if code_object_node.get_id() not in self._func_to_original_co_code:
+            self._func_to_original_co_code[code_object_node.get_id()] = code_object
         self._bp_status[aug_id] = bp_status
 
     def _get_callback_partial(self, callback, lineno, code_object, bp_status, filename, aug_id):
@@ -126,13 +156,26 @@ class Bdb(object):
     def clear_break(self, aug_id):
         try:
             cookie = self._cookies[aug_id]
+            code_object_node = self._cookies_to_func[cookie]
             status = self._bp_status[aug_id]
         except KeyError:
             return
 
-        cdbg_native.ClearConditionalBreakpoint(cookie)
+        try:
+            code_object = self._func_to_original_co_code[code_object_node.get_id()]
+        except KeyError:
+            code_object = code_object_node.obj
+
+        new_co_code = cdbg_native.ClearConditionalBreakpoint(cookie)
+
+        if sys.version_info >= (3, 11, 0):
+            new_code_object = code_object.replace(co_code=new_co_code)
+            self._set_code_object(code_object_node, new_code_object)
+
         status.disabled = True
+
         del self._cookies[aug_id]
+        del self._cookies_to_func[cookie]
         del self._bp_status[aug_id]
 
     def clear_all_breaks(self):
@@ -174,7 +217,7 @@ def callback(lineno, user_line, callback_object_id, bp_status, filename, pid, au
         import inspect
         frame = inspect.currentframe().f_back
 
-        callback_was_pickled = callback_object_id != id(frame.f_code) or pid != os.getpid()
+        callback_was_pickled = False#callback_object_id != id(frame.f_code) or pid != os.getpid()
 
         if not callback_was_pickled and frame and user_line:
             user_line(frame, filename, lineno=lineno, aug_id=aug_id)
